/** @(#) listpos/cpplistpos.cpp */
/**
* Implementa una lista basandose en <vector>.
* @author Omar Posada Villarreal
* @version 1.2, 10/12/2001 C++
* @version 1.1, 15/10/2001 getFirst(), getNext()
* @version 1.0, 03/10/2001
*/
#include "cpplistpos.h"

// constructor, ~, friend-----------------------------------------------------

/**
* Construye una lista estatica de taman~o fijo.
* @param theCapacity Numero de elementos. 10% del INT_MAX_VALUE.
* @see INT_MAX_VALUE
*/
template <class TObj>
CppListPos<TObj> :: CppListPos() : ListPos<TObj>() {
	//list = new vector<TObj>();
        list.reserve(INT_MAX_VALUE/10);
        itemCount = 0;
}


/** Destructor. Libera nodos. */
template <class TObj>
CppListPos<TObj> :: ~CppListPos() {
	delete list;
}

/**
* Regresa en un renglon a los elementos de la lista.
* En orden ascendente (del primero al ultimo) y separado por espacios.
* @return Flujo de los elementos.
*/
// oper unario friend no hace referencia a la clase
/* error Linker template <class TObj>
ostream& operator<< (ostream &outStr, const CppListPos<TObj> &list) { // 2 const
	ListNodePos<TObj>	*pCurrent = list.getFirstNode();
	outStr << "";	// inicializar

	if ( list.isEmpty() ) {
		return outStr;	//"";
	}
	// Mientras haya nodos, agregarlos a la cadena
	while (pCurrent != NULL) {
		//sList += pCurrent->item;
		outStr << pCurrent->getItem();
		pCurrent = pCurrent->getNext();
	}
	return outStr;
}
*/

// public---------------------------------------------------------------------

/**
* Cuenta el numero de elementos en la lista.
* @return Numero entero de elementos.
*/
template <class TObj>
int CppListPos<TObj> :: size() const {
	// itemCount se prefirio por herencia. opcional list.size()
	return itemCount;
}

/**
* Checa si la lista esta vacia.
* @return true No hay elementos en la lista
*/
template <class TObj>
bool CppListPos<TObj> :: isEmpty() const {
	return list.empty();
}

/**
* Checa si la lista esta llena.
* En teoria, la lista puede crecer indefinidamente,
* pero para conservar la consistencia con "size()" se limita
* al maximo soportado por un entero.
* @return true Se alcanzo el limite maximo.
*/
template <class TObj>
bool CppListPos<TObj> :: isFull() const {
	return (itemCount >= list.capacity() )
		? true : false;
}


/**
* Elimina todos los elementos de la lista. Libera memoria.
* No lanza "Exception".
*/
template <class TObj>
void CppListPos<TObj> :: removeAllElements() {
	itemCount = 0;
        list.clear();
}

/**
* Inserta un elemento al inicio de la lista.
* @param item Elemento.
* @throw FullListExceptionPos No hay capacidad de almacenamiento.
*/
template <class TObj>
void CppListPos<TObj> :: addFirst(const TObj item)
		throw (FullListExceptionPos) {
	if (isFull()) {
		throw FullListExceptionPos();
	}
        list.insert( list.begin(), item );
        ++itemCount;
}

/**
* Inserta un elemento al final de la lista.
* @param item Elemento.
* @throw FullListExceptionPos No hay capacidad de almacenamiento.
*/
template <class TObj>
void CppListPos<TObj> :: addLast(const TObj item) throw (FullListExceptionPos) {
	if (isFull()) {
		throw  FullListExceptionPos();
	}
        list.insert( list.end(), item );
        ++itemCount;
}

/**
* Toma el elemento al inicio de la lista y lo elimina.
* Libera memoria.
* @return item Elemento retirado.
* @throw EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj CppListPos<TObj> :: removeFirst() throw(EmptyListExceptionPos) {
	TObj removeItem;

	if (isEmpty()) {
		throw  EmptyListExceptionPos();
	}
        removeItem = list.front();
        list.erase( list.begin() );
	itemCount--;
	return removeItem;
}

/**
* Toma el elemento al final de la lista y lo elimina.
* Libera memoria.
* @return item Elemento retirado.
* @throw EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj CppListPos<TObj> :: removeLast() throw(EmptyListExceptionPos) {
	TObj removeItem;

	if (isEmpty()) {
		throw  EmptyListExceptionPos();
	}
        removeItem = list.back();
        list.pop_back();	//no usar erase( list.end() );
	itemCount--;
	return removeItem;
}

/**
* Regresa el contenido del elemento al final de la lista
* sin eliminarlo.
* @return item Elemento revisado.
* @throw EmptyListExceptionPos La lista esta vacia.
* @version 26/12/2001
*/
template <class TObj>
TObj CppListPos<TObj> :: getLast() throw(EmptyListExceptionPos) {
	if (isEmpty()) {
		throw  EmptyListExceptionPos();
	}
        TObj theLast = list.back();
        //cout << " Cpp::getLast: " << theLast;
        return theLast;
}

/**
* Regresa en un renglon a los elementos de la lista.
* En orden ascendente (del primero al ultimo) usando el
* metodo "TObj.toString()" y separado por espacios.
*/
// @return Cadena con los elementos
template <class TObj>
void CppListPos<TObj> :: print() const {
	if ( isEmpty() ) {
		return;	//"";
	}
        for (unsigned int i = 0; i < list.size(); ++i) {
		cout << list[i] << SEPARATOR;
        }
}

// private--------------------------------------------------------------------
// Fin------------------------------------------------------------------------
