/** @(#) listpos/dlinklistpos.cpp */
/**
* Implementa la lista doblemente ligada con almacenamiento dinamico.
* @author Omar Posada Villarreal
* @version 2.0, 02/01/2002 C++
* @version 1.0, 26/10/2001
*/
#include "dlinklistpos.h"

// constructor, ~, friend-----------------------------------------------------

/** Constructor. Inicializa a "first" y "last". */
template <class TObj>
DLinkListPos<TObj> :: DLinkListPos() : ListPos<TObj>() {
	//cout << "\n{ dllp";
        itemCount = 0;
	first = last = NULL;
	//cout << "\n  } dllp";
}

/** Destructor. Libera nodos. */
template <class TObj>
DLinkListPos<TObj> :: ~DLinkListPos() {
	removeAllElements();
}

// operator--------------------------------------------------------------------

/**
* Salida en un renglon a los elementos de la lista.
* En orden ascendente (del primero al ultimo) y separado por espacios.
* @param outStr Flujo de salida.
* @param list Lista.
* @return Flujo de salida con la lista.
*/
template <class TObj>
ostream &operator<<<TObj>(ostream &outStr, const DLinkListPos<TObj> &list) {
	DLinkNodePos<TObj>	*pCurrent;
        TObj	item;
	outStr << "";	// inicializar

        pCurrent = list.getFirstNode();	// necesita un miembro publico
	if ( list.isEmpty() ) {
		return outStr;	//"";
	}
	// Mientras haya nodos, agregarlos a la cadena
	while (pCurrent != NULL) {
	        item = pCurrent->getItem();
		outStr << item << " ";
		pCurrent = pCurrent->getRight();
	}
	return outStr;
}

// public---------------------------------------------------------------------

/**
* Cuenta el numero de elementos en la lista.
* @return Numero entero de elementos.
*/
template <class TObj>
int DLinkListPos<TObj> :: size() const {
	// error int DLinkListPos :: size() {
        //cout << "\nsize: " << itemCount;
	return itemCount;
}

/**
* Checa si la lista esta vacia.
* @return true No hay elementos en la lista
*/
template <class TObj>
bool DLinkListPos<TObj> :: isEmpty() const {
	// Si el primero apunta a NULL, se considera vacio,
	// No se checa a last
	return (first == NULL);
}

/**
* Checa si la lista esta llena.
* En teoria, la lista puede crecer indefinidamente,
* pero para conservar la consistencia con "size()" se limita
* al maximo soportado por un entero.
* @return true Se alcanzo el limite maximo.
*/
template <class TObj>
bool DLinkListPos<TObj> :: isFull() const {
	return (itemCount >= INT_MAX_VALUE);
}


/**
* Elimina todos los elementos de la lista. Libera memoria.
* No lanza "Exception".
*/
template <class TObj>
void DLinkListPos<TObj> :: removeAllElements() {
	// Mientras haya nodos, quitarlos
	while ( ! isEmpty() ) {
        	removeFirst();	// mas rapido que removeLast()
	}
	// redundante first = last = NULL; // no liberan memo
}

// hasta aqui ok

/**
* Inserta un elemento al inicio de la lista.
* @param item Elemento.
* @throws FullListExceptionPos No hay capacidad de almacenamiento.
*/
template <class TObj>
void DLinkListPos<TObj> :: addFirst(const TObj item)
		throw (FullListExceptionPos) {
	//cout << "\n{ addFirst item: " << item;
	DLinkNodePos<TObj>	*aux;
        ++itemCount;

	if (isEmpty()) {
		// No usar first = DLinkNodePos(); last = DLinkNodePos();
		// ya que se crearian dos copias sin relacion;
		first = last = new DLinkNodePos<TObj>(item);
                return;
	}
	aux	= first;
	first	= new DLinkNodePos<TObj>(item);
	first->setRight(aux);
	aux->setLeft(first);
	//cout << "\n  } addFirst";
}

/**
* Inserta un elemento al final de la lista.
* @param item Elemento.
* @throws FullListExceptionPos No hay capacidad de almacenamiento.
*/
template <class TObj>
void DLinkListPos<TObj> :: addLast(const TObj item)
		throw (FullListExceptionPos) {
	//cout << "\n{ addLast item: " << item;
	DLinkNodePos<TObj>	*aux;
        ++itemCount;

	if (isEmpty()) {
		first = last = new DLinkNodePos<TObj>(item);
                return;
	}
        aux = last;
	last	= new DLinkNodePos<TObj>(item);
	last->setLeft(aux);
	aux->setRight(last);
	//cout << "\n  } addLast";
}

/**
* Toma el elemento al inicio de la lista y lo elimina.
* Libera memoria.
* @return item Elemento retirado.
* @throws EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj DLinkListPos<TObj> :: removeFirst() throw(EmptyListExceptionPos) {
	// Error comun: olvidar poner "new throw Exception"
	TObj removeItem;	// no usar = NULL;

	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	removeItem = first->getItem();	// getItem();

	// Restablecer referencias de first y last
        DLinkNodePos<TObj> *aux = first;	// delete
	if (first == last) {
		first = last = NULL;
	} else {
		first = first->getRight();
	}
        delete aux;
        --itemCount;
	return removeItem;
}

/**
* Toma el elemento al final de la lista y lo elimina.
* Libera memoria.
* @return item Elemento retirado.
* @throws EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj DLinkListPos<TObj> :: removeLast() throw(EmptyListExceptionPos) {
	TObj removeItem; // No inicializar con = NULL si es string;

	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	removeItem = last->getItem();	// getItem();

	// Restablecer referencias de first y last
        DLinkNodePos<TObj> *aux = last;	// delete
	if (first == last) {
		first = last = NULL;
	} else {
	        last = last->getLeft();
	}
        delete aux;
        --itemCount;
	return removeItem;
}

/**
* Regresa el contenido del elemento al final de la lista
* sin eliminarlo.
* @return item Elemento revisado.
* @throws EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj DLinkListPos<TObj> :: getLast() throw(EmptyListExceptionPos) {
	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	return last->getItem();
}

/**
* Regresa en un renglon a los elementos de la lista.
* En orden ascendente (del primero al ultimo) usando el
* metodo "TObj.toString()" y separado por espacios.
*/
template <class TObj>
void DLinkListPos<TObj> :: print() const {
	cout << (*this);
}


//Solo de DLinkListPos---------------------------------------------------------

/**
* Regresa el contenido del elemento al inicio de la lista
* sin eliminarlo.
* @return item Elemento revisado.
* @throws EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj DLinkListPos<TObj> :: getFirst() const throw(EmptyListExceptionPos) {
	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	return first->getItem();
}

/**
* Regresa el primer nodo de la lista
* sin eliminarlo.
* @return Primer DLinkNodePos.
* @throws EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
DLinkNodePos<TObj> *  DLinkListPos<TObj> :: getFirstNode()
		const throw(EmptyListExceptionPos) {
	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	return first;
}

/**
* Regresa el ultimo nodo de la lista
* sin eliminarlo.
* @return Ultimo DLinkNodePos.
* @throws EmptyListExceptionPos La lista esta vacia.
*/
/*
template <class TObj>
DLinkNodePos<TObj> *  DLinkListPos<TObj> :: getLastNode()
		const throw(EmptyListExceptionPos) {
	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	return last;
}

/**
* Regresa el contenido del elemento anterior de la lista
* sin eliminarlo (izquierda).
* @param node Nodo del que se tomara al anterior (izquierda).
* @return DLinkNodePos con el nodo anterior.
* @throws EmptyListExceptionPos La lista esta vacia.
* El nodo es NULL.
*/
template <class TObj>
DLinkNodePos<TObj> *  DLinkListPos<TObj> :: getLeftNode(
		const DLinkNodePos<TObj> * node)
		const throw(EmptyListExceptionPos){
	if (node == NULL) {
		throw EmptyListExceptionPos("El nodo es NULL.");
	}
	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	return node->getLeft();
}


/**
* Regresa el contenido del siguiente elemento de la lista
* sin eliminarlo (derecha).
* @param node Nodo del que se tomara el siguiente (derecha).
* @return DLinkNodePos con el nodo siguiente.
* @throws EmptyListExceptionPos La lista esta vacia.
* El nodo es NULL.
*/
template <class TObj>
DLinkNodePos<TObj> *  DLinkListPos<TObj> :: getRightNode(
		const DLinkNodePos<TObj> * node)
		const throw(EmptyListExceptionPos){
	if (node == NULL) {
		throw EmptyListExceptionPos("El nodo es NULL.");
	}
	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	return node->getRight();
}

/**
* Busca la primer ocurrencia en que el objeto es igual.
* @param elem Objeto a comparar.
* @return un apuntador al nodo donde se encontro. NULL si no se encontro
* o la lista esta vacia.
*/
template <class TObj>
DLinkNodePos<TObj> * DLinkListPos<TObj> :: find(const TObj elem) const {
	DLinkNodePos<TObj> *current;
// JAVA

	if (isEmpty()) {
		return NULL;
	}

	current = first;
	do {
		if ( current->getItem() == elem ) {
			return current;
		}
		current = current->getRight();
	} while (current != NULL);

	return NULL;
}

/**
* Regresa el contenido del elemento actual de la lista
* sin eliminarlo.
* @return item Elemento revisado.
* @throws EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj DLinkListPos<TObj> :: get(const DLinkNodePos<TObj> * node)
		const throw(EmptyListExceptionPos) {
	if (isEmpty()) {
		throw EmptyListExceptionPos();
	}
	return node->getItem();
}

/**
* Elimina el nodo. Se asume que el nodo esta en la lista.
* @return item Elemento retirado.
* @throws EmptyListExceptionPos La lista esta vacia, o el nodo es NULL.
*/
template <class TObj>
TObj DLinkListPos<TObj> :: remove(DLinkNodePos<TObj> *node)
		throw(EmptyListExceptionPos) {
	TObj removeItem; // No usar = NULL;
	DLinkNodePos<TObj>	*leftNode;
	DLinkNodePos<TObj>	*rightNode;

	if ( isEmpty()  || (node == NULL) ) {
		throw EmptyListExceptionPos();
	}

	// pueden ser NULL
	//Log.p("-l r node");
	leftNode 	= node->getLeft();
	rightNode	= node->getRight();

	// Conecta lista
	//Log.p("- conecta");
	if (node == first) {
		first = node->getRight();
		rightNode->setLeft(NULL);
	} else if (node == last) {
		last = leftNode;
		leftNode->setRight(NULL);
	} else {
		//Log.p("DLList: " + this.toString() );
		leftNode->setRight(rightNode);
		rightNode->setLeft(leftNode);
	}

	// Elimina nodo
	//Log.p("- elim");
	removeItem = node->getItem();	// getItem();
	itemCount--;
	node = NULL;	//libera el nodo

	//Log.p("  } remove remItem: " + removeItem.toString() );
	return removeItem;
}

// Fin------------------------------------------------------------------------


