/** @(#) listpos/fixedlistpos.cpp */
/**
* Implementa una lista circular con arreglos de "Object" estatica.
* @author Omar Posada Villarreal
* @version 1.2, 10/12/2001 C++
* @version 1.1, 15/10/2001 getFirst(), getNext()
* @version 1.0, 03/10/2001
*/
#include "fixedlistpos.h"

// constructor, ~, friend-----------------------------------------------------

/**
* Construye una lista estatica de taman~o fijo.
* Para saber el taman~o predeterminado ver ListPos.INT_MAX_VALUE.
* @param theCapacity Numero de elementos.
* @see INT_MAX_VALUE
*/
template <class TObj>
FixedListPos<TObj> :: FixedListPos(int theCapacity) : ListPos<TObj>() {
        itemCount = 0;
        capacity = theCapacity;
	first = last = -1;

	list = new TObj[capacity]; // Crea el arreglo, error con pList[]
}


/** Destructor. Libera nodos. */
template <class TObj>
FixedListPos<TObj> :: ~FixedListPos() {
	delete list;
}

/**
* Regresa en un renglon a los elementos de la lista.
* En orden ascendente (del primero al ultimo) y separado por espacios.
* @return Flujo de los elementos.
*/
// oper unario friend no hace referencia a la clase
/* error Linker template <class TObj>
ostream& operator<< (ostream &outStr, const FixedListPos<TObj> &list) { // 2 const
	ListNodePos<TObj>	*pCurrent = list.getFirstNode();
	outStr << "";	// inicializar

	if ( list.isEmpty() ) {
		return outStr;	//"";
	}
	// Mientras haya nodos, agregarlos a la cadena
	while (pCurrent != NULL) {
		//sList += pCurrent->item;
		outStr << pCurrent->getItem();
		pCurrent = pCurrent->getNext();
	}
	return outStr;
}
*/

// public---------------------------------------------------------------------

/**
* Cuenta el numero de elementos en la lista.
* @return Numero entero de elementos.
*/
template <class TObj>
int FixedListPos<TObj> :: size() const {
	// error int FixedListPos :: size() {
	return itemCount;
}

/**
* Checa si la lista esta vacia.
* @return true No hay elementos en la lista
*/
template <class TObj>
bool FixedListPos<TObj> :: isEmpty() const {
	if ((first == -1) || (last == -1)) {
		//first = -1;	// Asegurarse que esta vacia
		//last = -1;
                removeAllElements(); // equivalente a lineas comentadas
		return true;
	} else {
		return false;
	}
}

/**
* Checa si la lista esta llena.
* En teoria, la lista puede crecer indefinidamente,
* pero para conservar la consistencia con "size()" se limita
* al maximo soportado por un entero.
* @return true Se alcanzo el limite maximo.
*/
template <class TObj>
bool FixedListPos<TObj> :: isFull() const {
	return ((last + 1) == first)
		? true : false;
}


/**
* Elimina todos los elementos de la lista. Libera memoria.
* No lanza "Exception".
*/
template <class TObj>
void FixedListPos<TObj> :: removeAllElements() {
	first = last = -1;
        // Es del destructor delete list;
}

/**
* Inserta un elemento al inicio de la lista.
* @param item Elemento.
* @throw FullListExceptionPos No hay capacidad de almacenamiento.
*/
template <class TObj>
void FixedListPos<TObj> :: addFirst(const TObj item)
		throw (FullListExceptionPos) {
	if (isFull()) {
		throw FullListExceptionPos();
	}
	// Si es vacio, apuntar al inicio del arreglo
	if (isEmpty()) {
		first = last = 0;
	} else {// Si el primero apunta a 0, irse al final
		if (first == 0) {
			first = capacity - 1;
		} else {
			first--;
		} // if first
	} // if empty
	list[first] = item;
        ++itemCount;
}

/**
* Inserta un elemento al final de la lista.
* @param item Elemento.
* @throw FullListExceptionPos No hay capacidad de almacenamiento.
*/
template <class TObj>
void FixedListPos<TObj> :: addLast(const TObj item)
		throw (FullListExceptionPos) {
	if (isFull()) {
		throw  FullListExceptionPos();
	}
	if (isEmpty()) {
		first = last = 0;
	} else {// Si esta al final del arreglo, ir al inicio
		if (last == (capacity - 1)) {
			last = 0;
		} else {
			last++;
		} // if last
	} // if isEmpty
	list[last] = item;
        ++itemCount;
}

/**
* Toma el elemento al inicio de la lista y lo elimina.
* Libera memoria.
* @return item Elemento retirado.
* @throw EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj FixedListPos<TObj> :: removeFirst() throw(EmptyListExceptionPos) {
	TObj removeItem;

	if (isEmpty()) {
		throw  EmptyListExceptionPos();
	}
	removeItem = list[first];
	itemCount--;

	// Si los dos apuntan al mismo, lista vacia
	if (last == first) {
		first = last = -1;
		return removeItem;
	} 

	// si se quita al final, ir al inicio
	if (first == (capacity-1)) {
		first = 0;
	} else {
		first++;
	}
	return removeItem;
}

/**
* Toma el elemento al final de la lista y lo elimina.
* Libera memoria.
* @return item Elemento retirado.
* @throw EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj FixedListPos<TObj> :: removeLast() throw(EmptyListExceptionPos) {
	TObj removeItem;

	if (isEmpty()) {
		throw  EmptyListExceptionPos();
	}
	removeItem = list[last];
	itemCount--;
	// Si los dos apuntan al mismo, lista vacia
	if (last == first) {
		first = last = -1;
		return removeItem;
	} 

	// si se quita al inicio, ir al final
	if (last == 0 ) {
		last = capacity - 1;
	} else {
		last--;
	}
	return removeItem;
}

/**
* Regresa el contenido del elemento al final de la lista
* sin eliminarlo.
* @return item Elemento revisado.
* @throw EmptyListExceptionPos La lista esta vacia.
*/
template <class TObj>
TObj FixedListPos<TObj> :: getLast() throw(EmptyListExceptionPos) {
	if (isEmpty()) {
		throw  EmptyListExceptionPos();
	}
	return list[last];
}

/**
* Regresa en un renglon a los elementos de la lista.
* En orden ascendente (del primero al ultimo) usando el
* metodo "TObj.toString()" y separado por espacios.
*/
// @return Cadena con los elementos
template <class TObj>
void FixedListPos<TObj> :: print() const {
	if ( isEmpty() ) {
		return;	//"";
	}

	unsigned int	i;	// Indice por numero de elementos
	int	k;	// Posicion a extraer

	for (i = 0, k = first; i < itemCount; i++) {
		cout << list[k] << SEPARATOR;

		// Ir al siguiente elemento
		if (k == (capacity - 1)) {
			k = 0;
		} else {
			k++;
		}
	}
}

// private--------------------------------------------------------------------
// Fin------------------------------------------------------------------------
