/** @(#) listpos/stackpos.cpp */
/**
* Implementa los metodos comunes a las pilas. 
* Representacion "FIFO", primero en entrar, primero en salir 
* ("First in, first out").
* Usa las implementaciones de "ListPos".
* @author Omar Posada Villarreal
* @version 2.0, 10/12/2001 C++
* @version 1.0, 03/10/2001
*/
#include "stackpos.h"

// constructor, ~, friend-----------------------------------------------------
/**
* Instancia o crea la lista a manejar.
* @param pTheList Lista concreta de un cierto tipo.
*/
template <class TObj>
StackPos<TObj> :: StackPos(ListPos<TObj> *pTheList) {
	pList = pTheList;
}

// Destructor predeterminado. La lista libera memoria

// public---------------------------------------------------------------------
/**
* Cuenta el numero de elementos en la cola.
* @return Numero entero de elementos.
*/
template <class TObj>
int StackPos<TObj> :: size() const {
	return pList->size();
}

/**
* Checa si la cola esta vacia.
* @return true No hay elementos en la cola
*/
template <class TObj>
bool StackPos<TObj> :: isEmpty() const {
	return pList->isEmpty();
}

/**
* Checa si la cola esta llena.
* @return true Se alcanzo el limite maximo.
*/
template <class TObj>
bool StackPos<TObj> :: isFull() const {
	return pList->isFull();
}

/**
* Regresa en un renglon a los elementos de la lista.
* En orden ascendente (del primero al ultimo) usando el
* metodo "Object.print()" y separado por espacios.
*/
template <class TObj>
void StackPos<TObj> :: print() const {
	pList->print();
}

/** Elimina todos los elementos de la cola. No lanza "Exception". */
template <class TObj>
void StackPos<TObj> :: removeAllElements() {
	pList->removeAllElements();
}

/**
* Agrega un elemento en el tope de la pila.
* @param item Elemento a insertar.
* @throws FullListExceptionPos No hay capacidad de almacenamiento.
*/
template <class TObj>
void StackPos<TObj> :: push(TObj item) throw (FullListExceptionPos) {
	pList->addLast(item);
}

/**
* Regresa el elemento en el tope de la pila.
* Uso: <pre>
*	itemString = (String)myStack.pop();
* </pre>
* @return item Elemento retirado.
* @throws EmptyListExceptionPos No hay elementos para retirar.
*/
template <class TObj>
TObj StackPos<TObj> :: pop() throw(EmptyListExceptionPos) {
	return pList->removeLast();
}

/**
* Regresa el elemento del tope de la pila sin retirarlo.
* @return Elemento a revisar.
* @throws EmptyListExceptionPos No hay elementos para revisar.
*/
template <class TObj>
TObj StackPos<TObj> :: peek() throw(EmptyListExceptionPos) {
	return pList->getLast();
}

// private--------------------------------------------------------------------
// Fin------------------------------------------------------------------------

